use eyes::parse;
use itertools::Itertools;

fn main() {
    let input = aoc::input!();
    println!("part 1: {}", part1(&input));
    println!("part 1: {}", part2(&input));
}

fn part1(input: &str) -> String {
    // parsing challenge..... yay......
    let (crates, instructions) = input.split_once("\n\n").unwrap();
    let mut stacks = crates
        .lines()
        .rev()
        .skip(1)
        .map(|line| {
            line.chars()
                .chunks(4)
                .into_iter()
                .map(|mut cr| (cr.next() == Some('[')).then(|| cr.next().unwrap()))
                .collect::<Vec<_>>()
        })
        .fold(vec![vec![]; 9], |mut stacks, crates| {
            for (i, c) in crates.into_iter().enumerate() {
                if let Some(c) = c {
                    stacks[i].push(c)
                }
            }
            stacks
        });

    // perform instructions
    instructions
        .lines()
        .map(|line| parse!(line, "move {} from {} to {}", usize, usize, usize))
        .for_each(|(n, from, to)| {
            (0..n).for_each(|_| {
                if let Some(c) = stacks[from - 1].pop() {
                    stacks[to - 1].push(c)
                }
            })
        });

    // get message
    stacks
        .into_iter()
        .filter_map(|mut stack| stack.pop())
        .collect::<String>()
}

fn part2(input: &str) -> String {
    let (crates, instructions) = input.split_once("\n\n").unwrap();
    let mut stacks = crates
        .lines()
        .rev()
        .skip(1)
        .map(|line| {
            line.chars()
                .chunks(4)
                .into_iter()
                .map(|mut cr| (cr.next() == Some('[')).then(|| cr.next().unwrap()))
                .collect::<Vec<_>>()
        })
        .fold(vec![vec![]; 9], |mut stacks, crates| {
            for (i, c) in crates.into_iter().enumerate() {
                if let Some(c) = c {
                    stacks[i].push(c)
                }
            }
            stacks
        });

    // perform instructions
    instructions
        .lines()
        .map(|line| parse!(line, "move {} from {} to {}", usize, usize, usize))
        .for_each(|(n, from, to)| {
            let len = stacks[from - 1].len();
            let split = stacks[from - 1].split_off(len - n);
            stacks[to - 1].extend(split);
        });

    // get message
    stacks
        .into_iter()
        .filter_map(|mut stack| stack.pop())
        .collect::<String>()
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn examples() {
        let example = r#"    [D]    
[N] [C]    
[Z] [M] [P]
 1   2   3 

move 1 from 2 to 1
move 3 from 1 to 3
move 2 from 2 to 1
move 1 from 1 to 2"#;
        assert_eq!(part1(example), "CMZ".to_string());
        assert_eq!(part2(example), "MCD".to_string());
    }
}
